/*
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/

package com.cesys.cebo.cebomsr.examples;

import java.io.IOException;
import java.util.List;

import com.cesys.cebo.cebomsr.AnalogInput;
import com.cesys.cebo.cebomsr.CurrentSource;
import com.cesys.cebo.cebomsr.Device;
import com.cesys.cebo.cebomsr.DeviceType;
import com.cesys.cebo.cebomsr.DigitalPort;
import com.cesys.cebo.cebomsr.LibraryInterface;
import com.cesys.cebo.cebomsr.Range;

/**
 * CeboMsr - Java example - Info.
 *
 * Print out various information about the device.
 */
public class InfoExample {

	/**
	 * Print various information to the console.
	 * @param device Device to use for test.
	 * @throws IOException
	 */
	private void printInformation(Device device) throws IOException {
		// Try to read the watchdog-timeout (introduced in firmware 1.4/CeboLC).
		String wd;
		try {
			// If this throws an exception, the device does not support a watchdog feature.
			long watchdog = device.getWatchdogTimeout();

			// If set to infinity, label it "Off".
			if (watchdog == 0xffffffffL)
				wd = "Off";
			else
				wd = String.format("%d ms", watchdog * 250);
		} catch (IOException ex) {
			// Contains detailed information why this is not supported.
			wd = ex.getMessage();
		}

		// Put out some device specific information.
		System.out.println("Device type: " + device.getDeviceType().getName());
		System.out.println("API vers: " + LibraryInterface.getApiVersion());
		System.out.println("Firmware ver: " + device.getFirmwareVersion());
		System.out.println("Identifier: " + device.getIdentifier());
		System.out.println("Serial number: " + device.getSerialNumber());
		System.out.println("Temperature: " + device.getTemperature());
		System.out.println("Watchdog: " + wd);

		// Get the real current of the reference current sources.
		for (CurrentSource source : device.getCurrentSources()) {
			System.out.println("Reference current of "
				+ source.getName() + ": "
				+ source.getReferenceCurrent() + " uA");
		}

		// Retrieve information about single ended input #0.
		// This works on all analog inputs and outputs.
		AnalogInput se0 = device.getSingleEndedInputs().get(0);
		System.out.println("Info for single ended input " + se0.getName() + ":");
		System.out.println("  Min. ICD: " + se0.getMinInterChannelDelay() + " us");
		System.out.println("  Current range: "
			+ se0.getRange().getMinValue() + " V to "
			+ se0.getRange().getMaxValue() + " V");
			System.out.println("Supported ranges:");
		for (int i = 0; i < se0.getSupportedRanges().size(); ++i) {
			Range range = se0.getSupportedRanges().get(i);
			System.out.println("    Range #" + i + " range: "
				+ range.getMinValue() + " V to "
				+ range.getMaxValue() + " V, "
				+ "Def. ICD: "
				+ se0.getDefaultInterChannelDelay(range) + " us");
		}

		// Get info for digital port #0.
		DigitalPort dp0 = device.getDigitalPorts().get(0);
		System.out.println("Info for digital port " + dp0.getName() + ":");
		System.out.println("  Count of I/O's: " + dp0.getIoCount());
		for (int i = 0; i < dp0.getIoCount(); ++i) {
			System.out.println("    I/O #" + i + ":" + dp0.getIoName(i));
		}
	}

	/**
	 * The examples main method.
	 */
	private void runExample() {
		try {
			// Search for devices ...
			List<Device> devices = LibraryInterface.enumerate(DeviceType.All);

			// If at least one has been found, use the first one ...
			if (!devices.isEmpty()) {
				Device device = devices.get(0);

				// Open device, nothing can be done without doing this.
				device.open();

				// Put out info.
				printInformation(device);

				// Finalize device usage, this free's up the device, so it can be used
				// again, including other applications.
				device.close();
			}
		}
		// These exceptions are currently used to report different problems ...
		catch (IllegalArgumentException ex) {
			ex.printStackTrace();
		}
		catch (IndexOutOfBoundsException ex) {
			ex.printStackTrace();
		}
		catch (IllegalStateException ex) {
			ex.printStackTrace();
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new InfoExample().runExample();
	}
}
