/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

package com.cesys.cebo.cebomsr.examples;

import java.io.IOException;
import java.util.List;

import com.cesys.cebo.cebomsr.Device;
import com.cesys.cebo.cebomsr.DeviceType;
import com.cesys.cebo.cebomsr.DigitalPort;
import com.cesys.cebo.cebomsr.LibraryInterface;

/**
 * CeboMsr - Java example - Single I/O.
 *
 * The I/O's of digital port #0 are read and mirrored to digital port #1.
 * (IO-0 - IO-7 are mirrored to IO-8 - IO-15).
 * The voltage at AI-0 is mirrored to AO-1 and AI-1 to AO-0 too.
 *
 * Does only work on CeboLC devices.
 */
public class SingleIoExample {

	/**
	 * Do very simple IO, digital as well as analog.
	 * @param device Device to use for test.
	 * @throws IOException
	 */
	private void singleIoTest(Device device) throws IOException {
		// Read digital port #0 and write result to digital port #1 (mirror pins).
		// At first, we need references to both ports.
		DigitalPort dp0 = device.getDigitalPorts().get(0);
		DigitalPort dp1 = device.getDigitalPorts().get(1);

		// Configure port first, all bits of digital port #0 as input (default)
		// and all bits of digital port #1 as output.
		dp1.setOutputEnableMask(0xff);

		// Read from #0 ...
		int value = dp0.read();

		// Write the value to #1 ...
		dp1.write(value);

		// Now some analog I/O, do it without any additional local references.
		// Read single ended #0 input voltage ...
		float voltageValue = device.getSingleEndedInputs().get(0).read();

		// Write value to analog output #1
		device.getAnalogOutputs().get(1).write(voltageValue);

		// .. or in one line: single ended #1 to analog output #0 ...
		device.getAnalogOutputs().get(0).write(device.getSingleEndedInputs().get(1).read());

		// Reset to power up defaults.
		device.resetDevice();
	}

	/**
	 * The examples main method.
	 */
	private void runExample() {
		try {
			// Search for devices ...
			List<Device> devices = LibraryInterface.enumerate(DeviceType.CeboLC);

			// If at least one has been found, use the first one ...
			if (!devices.isEmpty()) {
				Device device = devices.get(0);

				// Open device, nothing can be done without doing this.
				device.open();

				// *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
				// *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
				// *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
				singleIoTest(device);

				// Finalize device usage, this free's up the device, so it can be used
				// again, including other applications.
				device.close();
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new SingleIoExample().runExample();
	}
}
